-- just a bunch of functions:
--
-- applyForce, applyImpluse and move do exactly what their name says
--
-- phys1(who, dt) -- calculates next position (.disp) for moving objects
--
-- phys2(who, hash) -- applies .disp to .pos, so object actually moves
-- also, updates spatial hash info if given reference to it
--
-- for following functions: h - moving object, has to have .shape, .pos, .disp
--						start2 - .pos vector of top left corner of obj to check
--						sha2 - .shape vector (>v) of bject to check collision
--
-- top2bottom(h, start2, sha2) --- top of h to bottom of 2nd object
-- bottom2top(h, start2, sha2) --- bottom of h to top of 2nd object (aka h
--									standing on object
-- left2right(h, start2, sha2) --- left side of h to right side of obj
-- right2left(h, start2, sha2) --- right side of h to left side of obj
-- box2box(h, start2, sha2) --- if 2 boxes overlap, h has to have .pos & .shape
--
--
function applyForce(towho, dx, dy)
	towho.force.x = towho.force.x + dx
	towho.force.y = towho.force.y + dy
end

function applyImpulse(towho, dx, dy)
	towho.vel.x = towho.vel.x + dx
	towho.vel.y = towho.vel.y + dy or 0
end

function move(who, dx, dy)
	who.pos.x = who.pos.x + dx
	who.pos.y = who.pos.y + dy or 0
end

function nullVelocity(who, new_x, new_y)
	who.vel.x = new_x or 0
	who.vel.y = new_y or 0
end

function phys1(who, dt)
	who.vel = who.vel + who.force * dt
	who.force = who.gravity:clone()
	who.disp = who.pos + who.vel * dt
	-- apply somedamping for x axis movement
	who.vel.x = math.abs(who.vel.x) > eps and who.vel.x - who.vel.x * who.damp * dt or 0
end

function phys2(who, hash)
	if nil ~= hash then
		hash:update(who, who.disp)
	end
	who.pos = who.disp
end

function physics(who, hash, dt)
	-- in case we don't supply hash
	dt = dt or hash
	phys1(who, dt)
	--do collision & correction
	phys2(who, hash)
end

function standon(h, start2, sha2)
	if bottom2top(h, start2, sha2) then
		h.disp.y = start2.y - h.shape.y
		return true
	end
	return false
end

function nearwall(h, start2, sha2)
	if left2right(h, start2, sha2) then
		h.disp.x = start2.x + sha2.x
		return true
	end
	if right2left(h, start2, sha2) then
		h.disp.x = start2.x - h.shape.x
		return true
	end
	return false
end

function ceilinggravity(h, start2, sha2)
	if top2bottom(h, start2, sha2) then
		h.disp.y = start2.y + sha2.y
		return true
	end
	return false
end

function bottom2top(h, start2, sha2)
	local end1 = h.disp + h.shape
	local start1 = h.pos + h.shape
	-- h bottom to start2, sha2 top
	if end1.y > start2.y and start1.y <= start2.y then
		if end1.x > start2.x and end1.x - h.shape.x < start2.x + sha2.x then
			return true
		end
	end
	return false
end

function left2right(h, start2, sha2)
	local end1 = h.disp
	local start1 = h.pos
	-- left side of h to right side of start2, sha2
	if end1.x < start2.x + sha2.x and start1.x >= start2.x + sha2.x then
		if end1.y + h.shape.y > start2.y and end1.y < start2.y + sha2.y then
			return true
		end
	end
	return false
end

function right2left(h, start2, sha2)
	local end1 = h.disp
	local start1 = h.pos
	-- right side of h to left side of start2, sha2
	if end1.x + h.shape.x > start2.x and start1.x + h.shape.x <= start2.x then
		if end1.y + h.shape.y > start2.y and end1.y < start2.y + sha2.y then
			return true
		end
	end
	return false
end

function top2bottom(h, start2, sha2)
	local end1 = h.disp
	local start1 = h.pos
	-- h top to bottom of start2,sha2
	if end1.y < start2.y + sha2.y and start1.y >= start2.y + sha2.y then
		if end1.x + h.shape.x > start2.x and end1.x < start2.x + sha2.x then
			return true
		end
	end
	return false
end


-- h has to have pos vector (top left corner) ans shape (>v) vector
-- start2 - top left vector position
-- sha2 - vector shape from top left to bottom right corner
function box2box(h, start2, sha2)
	local s1 = h.pos
	if		s1.y + h.shape.y < start2.y or
			s1.y > start2.y + sha2.y or
			s1.x > start2.x + sha2.x or
			s1.x + h.shape.x < start2.x then
		return false
	end
	return true
end

function vec2vec(start1, sha1, start2, sha2)
	if		start1.y + sha1.y < start2.y or
			start1.y > start2.y + sha2.y or
			start1.x > start2.x + sha2.x or
			start1.x + sha1.x < start2.x then
			return false
	end
	return true
end
